package com.inet.ailink.receiver.service.impl;

import com.inet.ailink.receiver.common.enums.StatusCode;
import com.inet.ailink.receiver.common.exception.BizException;
import com.inet.ailink.receiver.common.utils.DateUtils;
import com.inet.ailink.receiver.common.vo.WebContext;
import com.inet.ailink.receiver.entity.BaseEntity;
import com.inet.ailink.receiver.mapper.BaseDao;
import com.inet.ailink.receiver.service.IBaseService;
//import com.inet.base.entity.BaseEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

public abstract class BaseServiceImpl<DAO extends BaseDao<E>, E extends BaseEntity> implements IBaseService<E> {
	protected Logger log = LoggerFactory.getLogger(getClass());
	
	protected abstract DAO getEntityDao();
	
  	public E get(Long id) throws BizException {
		return (E)getEntityDao().getById(id);
	}

  	
  	public int update(E entity) throws BizException {
  		int updateCount = 0;
  		if(null!=entity.getId()){
  			updateCount = save(entity);
  		}
  		if(updateCount==0){
			log.error("update count zero : "+entity.toString());
			throw new BizException(StatusCode.DAO_UPDATE_COUNT_ZERO_EXCEPTION);
  		}
  		return updateCount;
  	}
  	
	public int save(E entity) throws BizException {
		if(null!=entity){
			try{
				if(null==entity.getId()){
					entity.setDelStatus(0);//默认不删除： 0未删�? 1删除
					if(null != entity.getCreateUserId()){
						entity.setCreateUserId(entity.getCreateUserId());
					}else{
						entity.setCreateUserId(0L);
					}
					//entity.setCreatorName(WebContext.getWebContext().getUserName()==null?"system":WebContext.getWebContext().getUserName().toString());
					entity.setCreateTime(DateUtils.currentDate());
					//entity.setVersionNumber(1L);
					return getEntityDao().insert(entity); 
				}else{
					//更新的时候传入版本号，做版本号的校验，不传版本号则不做版本号检验
					/*if(entity.getVersionNumber() != null){
						E oldEntity = getEntityDao().getById(entity.getId());
						if(!entity.getVersionNumber().equals(oldEntity.getVersionNumber())){
							throw new BizException(StatusCode.CANT_MODIFY);
						}
					}*/
					/*entity.setUpdateUserId(WebContext.getWebContext().getUserId()==null?0L:WebContext.getWebContext().getUserId());
					entity.setUpdatorName(WebContext.getWebContext().getUserName()==null?"system":WebContext.getWebContext().getUserName().toString());*/
					entity.setUpdateTime(DateUtils.currentDate());
					return getEntityDao().update(entity);
				}
			}catch(Exception e){
				log.error("save failed.",e);
				throw new BizException(StatusCode.DAO_ERROR);
			}
		}
		return 0;
	}
	
	@Override
	@Transactional
  	public int save(List<E> entityList) throws BizException {
		if(null!=entityList){
			int count=0;
			try{
				List<E> insertList=new ArrayList<E>();
				List<E> updateList=new ArrayList<E>();
				for(E entity:entityList){
					if(null==entity.getId()){
						entity.setDelStatus(0);//默认不删除： 0未删�? 1删除
						entity.setCreateUserId(WebContext.getWebContext().getUserId());
						entity.setCreatorName(WebContext.getWebContext().getUserName()==null?"system":WebContext.getWebContext().getUserName().toString());
						entity.setCreateTime(DateUtils.currentDate());
						insertList.add(entity);
					}else{
						entity.setUpdateUserId(WebContext.getWebContext().getUserId());
						entity.setUpdatorName(WebContext.getWebContext().getUserName()==null?"system":WebContext.getWebContext().getUserName().toString());
						entity.setUpdateTime(DateUtils.currentDate());
						updateList.add(entity);
					}
				}
				count= getEntityDao().insertBatch(insertList)+getEntityDao().updateBatch(updateList);
			}catch(Exception e){
				log.error("save failed.",e);
				throw new BizException(StatusCode.DAO_ERROR.getCode(), StatusCode.DAO_ERROR.getMsg(),e);
			}
			return count;
		}
		return 0;
  	}
	
	public int remove(Long id) throws BizException {
		if(null!=id){
			try{
				return getEntityDao().delete(id);
			}catch(Exception e){
				log.error("remove failed.",e);
				throw new BizException(StatusCode.DAO_ERROR.getCode(), StatusCode.DAO_ERROR.getMsg(),e);
			}
		}
		return 0;
	}
	

	public List<E> selectListObjByAttribute(Object entity) throws BizException {
		if (entity == null) {
			return new ArrayList<E>();
		}
		return getEntityDao().getObjByAttribute(entity);
	}
	
}
 