 package com.inet.ailink.receiver.service.impl;

 import com.inet.ailink.receiver.common.enums.StatusCode;
 import com.inet.ailink.receiver.common.utils.Base64TeaUitls;
 import com.inet.ailink.receiver.common.utils.JsonUtils;
 import com.inet.ailink.receiver.common.utils.StringSelfUtil;
 import com.inet.ailink.receiver.common.vo.Response;
 import com.inet.ailink.receiver.entity.Device;
 import com.inet.ailink.receiver.entity.DeviceType;
 import com.inet.ailink.receiver.mapper.ISysDeviceInfoDao;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.stereotype.Service;
 import org.springframework.transaction.annotation.Transactional;

 import javax.servlet.http.HttpServletRequest;
 import java.nio.charset.StandardCharsets;
 import java.util.Date;

 @Service
 @Transactional(rollbackFor=Exception.class)
 public class SysDeviceInfoServiceImpl extends  BaseServiceImpl<ISysDeviceInfoDao,Device>
 {

     @Autowired
     ISysDeviceInfoDao sysDeviceInfoDao;

     @Override
     protected ISysDeviceInfoDao getEntityDao() {
         return sysDeviceInfoDao;
     }

     public static String byteArr2HexStr(byte[] arrB) {
         int iLen = arrB.length;
         // 每个byte用两个字符才能表示，所以字符串的长度是数组长度的两倍
         StringBuffer sb = new StringBuffer(iLen * 2);
         for (int i = 0; i < iLen; i++) {
             int intTmp = arrB[i];
             // 把负数转换为正数
             while (intTmp < 0) {
                 intTmp = intTmp + 256;
             }
             // 小于0F的数需要在前面补0
             if (intTmp < 16) {
                 sb.append("0");
             }
             sb.append(Integer.toString(intTmp, 16).toUpperCase());
             sb.append(" ");
         }
         return sb.toString();
     }
 //	public static byte[] hexStr2ByteArr(String strIn) {
 //		byte[] arrB = strIn.getBytes();
 //		int iLen = arrB.length;
 //
 //		// 两个字符表示一个字节，所以字节数组长度是字符串长度除以2
 //		byte[] arrOut = new byte[iLen / 2];
 //		for (int i = 0; i < iLen; i = i + 2) {
 //			String strTmp = new String(arrB, i, 2);
 //			arrOut[i / 2] = (byte) Integer.parseInt(strTmp, 16);
 //		}
 //		return arrOut;
 //	}

     public Response<Object> register(byte[] paramsByte, HttpServletRequest request){
 //		System.out.println("register(): " + byteArr2HexStr(paramsByte));
         Response<Object> result = new Response<Object>();
         //解析数据
         //设备地址
         String deviceMac = Base64TeaUitls.getStartToEndForHex(0, 5, paramsByte);
         //
         Integer cid = Integer.parseInt(Base64TeaUitls.getLowConactHighEndForInt(6, 7, paramsByte));
         Integer pid = Integer.parseInt(Base64TeaUitls.getLowConactHighEndForInt(8, 9, paramsByte));
         Integer vid = Integer.parseInt(Base64TeaUitls.getLowConactHighEndForInt(10, 11, paramsByte));
         //设备名
         String deviceName = Base64TeaUitls.getStartToEndForHex(12, 13, paramsByte);
         //产品型号
         String deviceType = Base64TeaUitls.getStartToEndForHex(14, 14, paramsByte);
 //		//硬件版本
 //		String deviceHardwareVersion = Base64TeaUitls.getStartToEndForInt(15, 15, paramsByte);
 //		//软件版本
 //		String deviceSoftwareVersion = Base64TeaUitls.getStartToEndForInt(16, 16, paramsByte);
 //		//用户版本
 //		String deviceUserAutoVersion = Base64TeaUitls.getStartToEndForInt(17, 17, paramsByte);
 //		//年月日
         String deviceDate = Base64TeaUitls.getStartToEndForInt(18, 20, paramsByte);
 //		System.out.println("deviceDate: " + deviceDate);
         //芯片id
         String chipId	= Base64TeaUitls.getStartToEndForHex(21, 26, paramsByte);
         //设备SN [12-14]WM05+[0-5]设备地址+[21-26]芯片id
         String deviceSN = deviceName +deviceType+deviceMac+chipId;
         deviceSN = deviceSN.replaceAll(":", "");
         //设备单位
         String deviceUnit = Base64TeaUitls.getStartToEndForInt(27, 27, paramsByte);
         //如果IMEI不为空，则使用IMEI当作设备的唯一标识
         String imei = null;
         if(paramsByte.length > 28){
             //设备IMEI长度
             int imeiLength = Integer.parseInt(Base64TeaUitls.getStartToEndForInt(28, 28, paramsByte));
             if(imeiLength > 0){
                 //获取imei
                 imei = Base64TeaUitls.getStartToEndForASIII(28+1, 28+imeiLength, paramsByte);
             }
         }

         //将解析出的数据，赋值
 //		Device device = new Device(deviceName, deviceMac, new Date(), cid, pid, vid, "",
 //				deviceType,deviceSoftwareVersion, deviceHardwareVersion, deviceUserAutoVersion, deviceDate,chipId,deviceSN,deviceUnit);
         byte[] name = new byte[2];
         name[0] = paramsByte[12];
         name[1] = paramsByte[13];
         String moduleName = new String(name, StandardCharsets.UTF_8);
 //		System.out.println("moduleName: " + moduleName);
         int type = paramsByte[14] & 0xFF;
         String moduleType = (type<10?("0"):"") + type;
 //		System.out.println("moduleType: " + moduleType);
         int hardware = paramsByte[15] & 0xFF;
         String hardwareVer = "H" + hardware;
 //		System.out.println("hardwareVer: " + hardwareVer);
         int software = paramsByte[16] & 0xFF;
         String softwareVer = "S" + String.valueOf(software / 10) + "." + String.valueOf(software % 10);
 //		System.out.println("softwareVer: " + softwareVer);
         int custom = paramsByte[17] & 0xFF;
         String customVer = "." + String.valueOf(custom);
 //		System.out.println("customVer: " + customVer);
         int year = paramsByte[18] & 0xFF;
         int month = paramsByte[19] & 0xFF;
         int day = paramsByte[20] & 0xFF;
         String dateStr = "_20" + (year<10?("0"+year):year) + (month<10?("0"+month):month) + (day<10?("0"+day):day);
 //		System.out.println("dateStr: " + dateStr);
         String version = moduleName + moduleType + hardwareVer + softwareVer + customVer + dateStr;
 //		System.out.println("version: " + version);
         Device device = new Device(null, deviceMac, new Date(), cid, pid, vid, "",
                 null, version, null, null, null, chipId, deviceSN, deviceUnit);
         //如果IMEI不为空，则使用IMEI当作设备的唯一标识
         if(imei != null && !StringSelfUtil.isEmpty(imei)){
             device.setMac(imei.replaceAll(":", ""));
             device.setDeviceSN(imei.replaceAll(":", ""));
         }
         System.out.println(JsonUtils.toJson(device));
         //注册设备
         result = saveDevice(device);
         return result;
     }

     //保存设备
     public Response<Object> saveDevice(Device device){
         Response<Object> result = new Response<Object>();
         int updateResult = 0;
         if(device.getDeviceId() == null){//新增设备
             //通过pid、vid、cid、mac查询当前设备是否存在,如果存在，则直接返回，否则，新增设备
             Device params = new Device();
             params.setPid(device.getPid());
             params.setVid(device.getVid());
             params.setCid(device.getCid());
             params.setMac(device.getMac());
             Device deviceResult = sysDeviceInfoDao.getLastObjByAttribute(params);
             if(deviceResult != null){
                 result.setStatus(StatusCode.SUCCESS.getCode());
                 result.setData(deviceResult.getDeviceId());
                 return result;
             }else{
                 updateResult = sysDeviceInfoDao.insert(device);
             }
         }else{//修改设备
             updateResult = sysDeviceInfoDao.update(device);
         }
         if(updateResult>0){
             result.setStatus(StatusCode.SUCCESS.getCode());
             result.setData(device.getDeviceId());
         }else{
             result.setStatus(StatusCode.FAIL.getCode());
         }
         return result;
     }

 }